@@ -28,10 +28,12 @@
 
 #include "gclue-locator.h"
 
-#include "gclue-static-source.h"
-#include "gclue-wifi.h"
 #include "gclue-config.h"
 
+#if GCLUE_USE_WIFI_SOURCE
+#include "gclue-wifi.h"
+#endif
+
 #if GCLUE_USE_3G_SOURCE
 #include "gclue-3g.h"
 #endif
@@ -48,6 +50,14 @@
 #include "gclue-nmea-source.h"
 #endif
 
+#if GCLUE_USE_STATIC_SOURCE
+#include "gclue-static-source.h"
+#endif
+
+#if GCLUE_USE_IP_SOURCE
+#include "gclue-ip.h"
+#endif
+
 /* This class is like a master location source that hides all individual
  * location sources from rest of the code
  */
@@ -207,8 +217,10 @@ refresh_available_accuracy_level (GClueLocator *locator)
                         (locator->priv->sources,
                          (GCompareFunc) compare_accuracy_level);
 
-        new = gclue_location_source_get_available_accuracy_level
-                        (GCLUE_LOCATION_SOURCE (locator->priv->sources->data));
+        new = locator->priv->sources ?
+                gclue_location_source_get_available_accuracy_level
+                                (GCLUE_LOCATION_SOURCE (locator->priv->sources->data)) :
+                GCLUE_ACCURACY_LEVEL_NONE;
 
         existing = gclue_location_source_get_available_accuracy_level
                         (GCLUE_LOCATION_SOURCE (locator));
@@ -400,7 +412,6 @@ gclue_locator_constructed (GObject *object)
         GClueLocator *locator = GCLUE_LOCATOR (object);
         GClueLocationSource *submit_source = NULL;
         GClueConfig *gconfig = gclue_config_get_singleton ();
-        GClueWifi *wifi = NULL;
         GList *node;
         GClueMinUINT *threshold;
 
@@ -420,20 +431,13 @@ gclue_locator_constructed (GObject *object)
                                                         cdma);
         }
 #endif
+#if GCLUE_USE_WIFI_SOURCE
         if (gclue_config_get_enable_wifi_source (gconfig)) {
-                wifi = gclue_wifi_get_singleton (locator->priv->accuracy_level);
-        } else {
-                if (gclue_config_get_enable_static_source (gconfig)) {
-                        g_debug ("Disabling GeoIP-only source since static source is enabled");
-                } else {
-                        /* City-level accuracy will give us GeoIP-only source */
-                        wifi = gclue_wifi_get_singleton (GCLUE_ACCURACY_LEVEL_CITY);
-                }
-        }
-        if (wifi) {
+                GClueWifi *wifi = gclue_wifi_get_singleton (locator->priv->accuracy_level);
                 locator->priv->sources = g_list_append (locator->priv->sources,
                                                         wifi);
         }
+#endif
 #if GCLUE_USE_MODEM_GPS_SOURCE
         if (gclue_config_get_enable_modem_gps_source (gconfig)) {
                 GClueModemGPS *gps = gclue_modem_gps_get_singleton ();
@@ -455,7 +459,7 @@ gclue_locator_constructed (GObject *object)
 
         }
 #endif
-
+#if GCLUE_USE_STATIC_SOURCE
         if (gclue_config_get_enable_static_source (gconfig)) {
                 GClueStaticSource *static_source;
 
@@ -464,6 +468,19 @@ gclue_locator_constructed (GObject *object)
                 locator->priv->sources = g_list_append (locator->priv->sources,
                                                         static_source);
         }
+#endif
+#if GCLUE_USE_IP_SOURCE
+        if (gclue_config_get_enable_ip_source (gconfig)) {
+                GClueIp *source = gclue_ip_get_singleton ();
+                locator->priv->sources = g_list_append (locator->priv->sources,
+                                                        source);
+        }
+#endif
+
+        if (locator->priv->sources == NULL) {
+                g_warning ("No sources enabled in configuration, "
+                           "location not available");
+        }
 
         for (node = locator->priv->sources; node != NULL; node = node->next) {
                 g_signal_connect (G_OBJECT (node->data),
@@ -569,7 +586,9 @@ gclue_locator_stop (GClueLocationSource *source)
         GList *node;
         GClueLocationSourceStopResult base_result;
 
-        g_return_val_if_fail (GCLUE_IS_LOCATOR (source), FALSE);
+        g_return_val_if_fail (GCLUE_IS_LOCATOR (source),
+                              GCLUE_LOCATION_SOURCE_STOP_RESULT_FAILED);
+
         locator = GCLUE_LOCATOR (source);
 
         base_class = GCLUE_LOCATION_SOURCE_CLASS (gclue_locator_parent_class);
